% NGLIGEN    Ng and Li's General Test-Problem Generator (version 2.0)
%
% NGLIGEN uses the parameters stored in "nglinnnn.par" to generate an
% instant of Ng and Li's test problems.
% Here nnnn is the problem-ID between 1000 and 9999.
% It is also the random seed for generating the problem.
%
% The file "nglinnnn.par" must contain the following items and 
% appear in the following order:
% - n (= the number of decision variables and the number of subproblems)
%   It is a positive integer.
% - nDiff (= the number of relatively difficult subproblems)
%   It is a positive integer not greater than n.
%   The remaining (n - nDiff) subproblems are then relatively easy 
%   subproblems.  
%   Level 0 problem: nDiff=0
%   Level 1 problem: nDiff=n/2 if n is even, nDiff=(n+1)/2 is n is odd
%   Level 2 problem: nDiff=n
% - aLB and aUB (= lower and upper bounds for a(i), respectively)
%   In theory, a(i) > 0.
%   In practice, we generate a(i) uniformly in [aLB, aUB] such that
%   1 <= aLB < aUB and aUB/aLB <= 10.
% - pLB and pUB (= lower and upper bounds for p(i), respectively)
%   In theory, p(i) is a real number.
%   In practice, we generate p(i) uniformly in [pLB, pUB] such that
%   pUB = -pLB > 0.
% - qLB and qUB
%   In thoery, q(i) is a real number such that q(i) < p(i)^2
%   In practice, we generate q(i) uniformly in [qLB, qUB] such that
%   qLB < qUB <= -1.
% - alphaPrime
%   In theory,
%      -p(i)-2*r(i) < alpha(i) < -p(i)-r(i)*sqrt(3)  OR
%      -p(i)+r(i)*sqrt(3) < alpha(i) < -p(i)+2*r(i)
%   In practice, to avoid the boundary points, we generate alpha(i)
%   uniformly in a fraction alphaPrime (e.g. 0.95) of the middle parts 
%   of the open intervals.
% - dLB and dUB
%   In theory, d(i) > 0.
%   In practice, we generate d(i) uniformly in [dLB, dUB]
%   such that 0.1 <= dLB < dUB and dUB/dLB <= 10.
% - deltaLB and deltaUB
%   In theory deltaL(i) > 0 and deltaR(i) > 0
%   In practice, we generate deltaL(i) and deltaR(i) uniformly in 
%   [deltaLB, deltaUB] such that 0.1 <= deltaLB < deltaUB <= 1.





function ngligen(ID)



StartTime = cputime;

if nargin == 0
  error('ngligen(ID), where 1000 <= ID <= 9999');
elseif (ID < 1000) || (ID > 9999)
  error(['To generate a user-defined problem, ',...
         'please use an ID between 1000 and 9999.']);
else
  NL = read_para(ID);
  fprintf('Generating user-defined test-problem number %d... ', NL.id);
  generate_rand(NL);
  nglimain(NL);
  fprintf('Done.\n');
end

EndTime = cputime;
fprintf('Time Elapsed = %f\n', EndTime-StartTime);





function NL = read_para(ID)


NL.id = ID;

fid = fopen(['ngli', num2str(NL.id,'%04u'), '.par'], 'r');
if fid <= 0
  error(['The parameter file "ngli', num2str(NL.id,'%04u'),...
         '.par" does not exist.']);
end


% Read n (>= 1)
%
NL.n = fscanf(fid, '%d', 1);
if NL.n < 2
  fclose(fid);
  error('Dimension n >= 1.');
end


% Read nDiff
%
NL.nDiff = fscanf(fid, '%d', 1);
if (NL.nDiff < 0) || (NL.nDiff > NL.n)
  fclose(fid);
  error('Number of difficulty subproblems in [0,n].');
end


% Read aLB and aUB
%
NL.aLB = fscanf(fid, '%f', 1);
NL.aUB = fscanf(fid, '%f', 1);
if NL.aLB >= NL.aUB
  fclose(fid);
  error('aLB <= a(i) <= aUB.');
end


% Read pLB and pUB
%
NL.pLB = fscanf(fid, '%f', 1);
NL.pUB = fscanf(fid, '%f', 1);
if NL.pLB >= NL.pUB
  fclose(fid);
  error('pLB <= p(i) <= pUB.');
end


% Read qLB and qUB
%
NL.qLB = fscanf(fid, '%f', 1);
NL.qUB = fscanf(fid, '%f', 1);
if (NL.qLB >= NL.qUB) || (NL.qUB > 0)
  fclose(fid);
  error('qLB <= q(i) <= qUB < 0.');
end


% Read alphaPrime
%
NL.alphaPrime = fscanf(fid, '%f', 1);
if (NL.alphaPrime < 0.5) || (NL.alphaPrime >= 1)
  fclose(fid);
  error('The fraction alphaPrime in [0.5,1).');
end


% Read dLB and dUB
%
NL.dLB = fscanf(fid, '%f', 1);
NL.dUB = fscanf(fid, '%f', 1);
if NL.dLB >= NL.dUB
  fclose(fid);
  error('dLB <= d(i) <= dUB.');
end


% Read deltaLB and deltaUB
%
NL.deltaLB = fscanf(fid, '%f', 1);
NL.deltaUB = fscanf(fid, '%f', 1);
if NL.deltaLB >= NL.deltaUB
  fclose(fid);
  error('deltaLB <= delta(i) <= deltaUB.');
end





function generate_rand(NL)

RandStream.setDefaultStream(RandStream('mt19937ar','seed',NL.id));

AllRand = rand(8*NL.n, 1);

fid = fopen(['ngli', num2str(NL.id,'%04u'), '.rn'], 'w');
fprintf(fid, '% 19.15e\n', AllRand);
fclose(fid);
