% NGLIOBJ    Ng and Li's Standard Objective Function (version 2.0)
%
% F = NGLIOBJ(X,NL) calculates the objective function value of the
% Ng and Li's test problem at X.  The parameters of the test problem 
% are stored in the structure array NL with fields:
%   n: integer,
%   a, p, q, s, d, v : column n-vectors, and
%   val: real
% For standard test problems 1--300, NL are stored in the m-files
% NLnnn.M, where nnn is the problem ID.
% For example, we can find the objective function value of the problem 
% number 1 at the point X = [0;0] as follows: 
%    fval = ngliobj([0;0], ngli000).
% For example, we can have a contour plot of the problem number 1 as
% follows:
%    NL = ngli001;
%    y1 = -15:0.1:15;
%    y2 = -15:0.1:15;
%    [Y1,Y2] = meshgrid(y1,y2);
%    G = zeros(size(Y1));
%    for col = 1:length(y1)
%      for row = 1:length(y2)
%        G(row,col) = ngliobj([Y1(row,col);Y2(row,col)], NL);
%      end
%    end
%    contour(Y1,Y2,G,50)
%    axis equal image
%
% [F,DF] = NGLIOBJ(X,NL) calculates the objective function value and
% the gradient column vector of the Ng and Li's test problem at X 
% with parameters in NL.
%
% [F,DF,DDF] = NGLIOBJ(X,NL) calculates the objective function value,
% the gradient column vector and the Hessian matrix of the Ng and Li's
% test problem at X with parameters in NL.
% For example, we can find a local minimizer and its value for the 
% problem number 1 by FMINUNC with initial point [0;0] as follows:
%    fun      = @(x)ngliobj(x, ngli001);
%    x0       = [0;0];
%    options  = optimset('GradObj','on', 'Hessian','on');
%    [x,fval] = fminunc(fun, x0, options);



function [g, dg, ddg] = ngliobj(y, NL, non0)



if nargin == 2
  non0 = true;
elseif nargin ~= 3
  error('[g, dg, ddg] = ngliobj(y, NL, non0)');
end



D = diag(NL.d);
H = eye(NL.n) - 2 * (NL.v * NL.v.');

x = D * H * y;
g = NL.a.' * (x.^4 + 4*NL.p.*x.^3 + 6*NL.q.*x.^2 + NL.s.*x);
if ~non0
  g = g - NL.val;
end

if nargout >= 2
  df = NL.a .* (4*x.^3 + 12*NL.p.*x.^2 + 12*NL.q.*x + NL.s);
  dg = H * D * df;
end

if nargout >= 3
  ddf = diag( 12*NL.a.*(x.^2 + 2*NL.p.*x + NL.q) );
  ddg = H * D * ddf * D * H;
end
